@@ -32,9 +32,16 @@
 
 */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
 #include <speex/speex_bits.h>
 #include "misc.h"
 
+/** Maximum size of the bit-stream (for fixed-size allocation) */
+#define MAX_BYTES_PER_FRAME 2000
+
 void speex_bits_init(SpeexBits *bits)
 {
    int i;
@@ -95,7 +102,7 @@ void speex_bits_read_from(SpeexBits *bits, char *bytes, int len)
    int i;
    if (len > bits->buf_size)
    {
-      speex_warning_int("Packet is larger than allocated buffer: ", len);
+      speex_warning_int("Packet if larger than allocated buffer: ", len);
       if (bits->owner)
       {
          char *tmp = (char*)speex_realloc(bits->bytes, len);
@@ -136,9 +143,9 @@ void speex_bits_read_whole_bytes(SpeexBits *bits, char *bytes, int len)
 {
    int i,pos;
 
-   if ((bits->nbBits>>3)+len+1 > bits->buf_size)
+   if (((bits->nbBits+7)>>3)+len > bits->buf_size)
    {
-      speex_warning_int("Packet is larger than allocated buffer: ", len);
+      /* Packet is larger than allocated buffer */
       if (bits->owner)
       {
          char *tmp = (char*)speex_realloc(bits->bytes, (bits->nbBits>>3)+len+1);
@@ -167,7 +174,7 @@ int speex_bits_write(SpeexBits *bits, char *bytes, int max_len)
 {
    int i;
    int bytePtr, bitPtr, nbBits;
-
+   
    /* Insert terminator, but save the data so we can put it back after */
    bitPtr=bits->bitPtr;
    bytePtr=bits->bytePtr;
@@ -321,24 +328,12 @@ int speex_bits_peek(SpeexBits *bits)
 
 void speex_bits_advance(SpeexBits *bits, int n)
 {
-   int nbytes, nbits;
-
-   if ((bits->bytePtr<<3)+bits->bitPtr+n>bits->nbBits)
+    if (((bits->bytePtr<<3)+bits->bitPtr+n>bits->nbBits) || bits->overflow){
       bits->overflow=1;
-   if (bits->overflow)
       return;
-
-   nbytes = n >> 3;
-   nbits = n & 7;
-   
-   bits->bytePtr += nbytes;
-   bits->bitPtr += nbits;
-   
-   if (bits->bitPtr>7)
-   {
-      bits->bitPtr-=8;
-      bits->bytePtr++;
-   }
+    }
+   bits->bytePtr += (bits->bitPtr+n) >> 3; /*divide by 8*/
+   bits->bitPtr = (bits->bitPtr+n) & 7;    /* modulo by 8*/
 }
 
 int speex_bits_remaining(SpeexBits *bits)
