@@ -37,19 +37,13 @@
 
 #ifndef SPEEX_VERSION
 #define SPEEX_MAJOR_VERSION 1
-#define SPEEX_MINOR_VERSION 0
-#define SPEEX_MICRO_VERSION 4
+#define SPEEX_MINOR_VERSION 1
+#define SPEEX_MICRO_VERSION 6
 #define SPEEX_EXTRA_VERSION ""
-#define SPEEX_VERSION "speex-1.0.4"
+#define SPEEX_VERSION "speex-1.1.6"
 #endif
 
-/*Disable some warnings on VC++*/
-#ifdef _MSC_VER
-#pragma warning(disable : 4244)
-#pragma warning(disable : 4305)
-#endif
-
-#define VERY_SMALL 1e-15
+#include "arch.h"
 
 #ifndef RELEASE
 void print_vec(float *vec, int len, char *name);
@@ -80,7 +74,7 @@ void speex_warning(char *str);
 
 void speex_warning_int(char *str, int val);
 
-void speex_rand_vec(float std, float *data, int len);
+void speex_rand_vec(float std, spx_sig_t *data, int len);
 
 float speex_rand(float std);
 
