@@ -32,6 +32,10 @@
 
 */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
 #include "modes.h"
 #include "ltp.h"
 #include "quant_lsp.h"
@@ -62,8 +66,6 @@ extern const signed char exc_10_16_table[];
 extern const signed char exc_20_32_table[];
 extern const signed char hexc_10_32_table[];
 
-static int nb_mode_query(const void *mode, int request, void *ptr);
-static int wb_mode_query(const void *mode, int request, void *ptr);
 
 /* Parameters for Long-Term Prediction (LTP)*/
 static const ltp_params ltp_params_nb = {
@@ -688,192 +690,9 @@ const SpeexMode speex_nb_48k_mode = {
 
 #endif
 
-
-
-void *speex_encoder_init(const SpeexMode *mode)
-{
-   return mode->enc_init(mode);
-}
-
-void *speex_decoder_init(const SpeexMode *mode)
-{
-   return mode->dec_init(mode);
-}
-
-void speex_encoder_destroy(void *state)
-{
-   (*((SpeexMode**)state))->enc_destroy(state);
-}
-
-void speex_decoder_destroy(void *state)
-{
-   (*((SpeexMode**)state))->dec_destroy(state);
-}
-
-
-
-int speex_encode_native(void *state, spx_word16_t *in, SpeexBits *bits)
-{
-   return (*((SpeexMode**)state))->enc(state, in, bits);
-}
-
-int speex_decode_native(void *state, SpeexBits *bits, spx_word16_t *out)
-{
-   return (*((SpeexMode**)state))->dec(state, bits, out);
-}
-
-
-
-#ifdef FIXED_POINT
-
-int speex_encode(void *state, float *in, SpeexBits *bits)
-{
-   int i;
-   int N;
-   short short_in[MAX_IN_SAMPLES];
-   speex_encoder_ctl(state, SPEEX_GET_FRAME_SIZE, &N);
-   for (i=0;i<N;i++)
-   {
-      if (in[i]>32767.f)
-         short_in[i] = 32767;
-      else if (in[i]<-32768.f)
-         short_in[i] = -32768;
-      else
-         short_in[i] = (short)floor(.5+in[i]);
-   }
-   return (*((SpeexMode**)state))->enc(state, short_in, bits);
-}
-
-int speex_encode_int(void *state, short *in, SpeexBits *bits)
-{
-   return (*((SpeexMode**)state))->enc(state, in, bits);
-}
-
-int speex_decode(void *state, SpeexBits *bits, float *out)
-{
-   int i, ret;
-   int N;
-   float short_out[MAX_IN_SAMPLES];
-   speex_decoder_ctl(state, SPEEX_GET_FRAME_SIZE, &N);
-   ret = (*((SpeexMode**)state))->dec(state, bits, short_out);
-   for (i=0;i<N;i++)
-      out[i] = short_out[i];
-   return ret;
-}
-
-int speex_decode_int(void *state, SpeexBits *bits, short *out)
-{
-   return (*((SpeexMode**)state))->dec(state, bits, out);
-}
-
-#else
-
-int speex_encode(void *state, float *in, SpeexBits *bits)
-{
-   return (*((SpeexMode**)state))->enc(state, in, bits);
-}
-
-int speex_encode_int(void *state, short *in, SpeexBits *bits)
-{
-   int i;
-   int N;
-   float float_in[MAX_IN_SAMPLES];
-   speex_encoder_ctl(state, SPEEX_GET_FRAME_SIZE, &N);
-   for (i=0;i<N;i++)
-      float_in[i] = in[i];
-   return (*((SpeexMode**)state))->enc(state, float_in, bits);
-}
-
-int speex_decode(void *state, SpeexBits *bits, float *out)
-{
-   return (*((SpeexMode**)state))->dec(state, bits, out);
-}
-
-int speex_decode_int(void *state, SpeexBits *bits, short *out)
-{
-   int i;
-   int N;
-   float float_out[MAX_IN_SAMPLES];
-   int ret;
-   speex_decoder_ctl(state, SPEEX_GET_FRAME_SIZE, &N);
-   ret = (*((SpeexMode**)state))->dec(state, bits, float_out);
-   for (i=0;i<N;i++)
-   {
-      if (float_out[i]>32767.f)
-         out[i] = 32767;
-      else if (float_out[i]<-32768.f)
-         out[i] = -32768;
-      else
-         out[i] = (short)floor(.5+float_out[i]);
-   }
-   return ret;
-}
-#endif
-
-
-
-int speex_encoder_ctl(void *state, int request, void *ptr)
-{
-   return (*((SpeexMode**)state))->enc_ctl(state, request, ptr);
-}
-
-int speex_decoder_ctl(void *state, int request, void *ptr)
-{
-   return (*((SpeexMode**)state))->dec_ctl(state, request, ptr);
-}
-
-
-
-static int nb_mode_query(const void *mode, int request, void *ptr)
-{
-   const SpeexNBMode *m = (const SpeexNBMode*)mode;
-   
-   switch (request)
-   {
-   case SPEEX_MODE_FRAME_SIZE:
-      *((int*)ptr)=m->frameSize;
-      break;
-   case SPEEX_SUBMODE_BITS_PER_FRAME:
-      if (*((int*)ptr)==0)
-         *((int*)ptr) = NB_SUBMODE_BITS+1;
-      else if (m->submodes[*((int*)ptr)]==NULL)
-         *((int*)ptr) = -1;
-      else
-         *((int*)ptr) = m->submodes[*((int*)ptr)]->bits_per_frame;
-      break;
-   default:
-      speex_warning_int("Unknown nb_mode_query request: ", request);
-      return -1;
-   }
-   return 0;
-}
-
-static int wb_mode_query(const void *mode, int request, void *ptr)
-{
-   const SpeexSBMode *m = (const SpeexSBMode*)mode;
-
-   switch (request)
-   {
-   case SPEEX_MODE_FRAME_SIZE:
-      *((int*)ptr)=2*m->frameSize;
-      break;
-   case SPEEX_SUBMODE_BITS_PER_FRAME:
-      if (*((int*)ptr)==0)
-         *((int*)ptr) = SB_SUBMODE_BITS+1;
-      else if (m->submodes[*((int*)ptr)]==NULL)
-         *((int*)ptr) = -1;
-      else
-         *((int*)ptr) = m->submodes[*((int*)ptr)]->bits_per_frame;
-      break;
-   default:
-      speex_warning_int("Unknown wb_mode_query request: ", request);
-      return -1;
-   }
-   return 0;
-}
-
-
 int speex_mode_query(const SpeexMode *mode, int request, void *ptr)
 {
    return mode->query(mode->mode, request, ptr);
 }
+
+
