@@ -36,8 +36,8 @@
 #ifndef MODES_H
 #define MODES_H
 
-#include "speex.h"
-#include "speex_bits.h"
+#include <speex/speex.h>
+#include <speex/speex_bits.h>
 #include "misc.h"
 
 #define NB_SUBMODES 16
@@ -141,11 +141,14 @@ typedef struct SpeexSBMode {
    int     defaultSubmode; /**< Default sub-mode to use when encoding */
    int     low_quality_map[11]; /**< Mode corresponding to each quality setting */
    int     quality_map[11]; /**< Mode corresponding to each quality setting */
-   float   (*vbr_thresh)[11];
+   const float (*vbr_thresh)[11];
    int     nb_modes;
 } SpeexSBMode;
 
 int speex_encode_native(void *state, spx_word16_t *in, SpeexBits *bits);
 int speex_decode_native(void *state, SpeexBits *bits, spx_word16_t *out);
 
+int nb_mode_query(const void *mode, int request, void *ptr);
+int wb_mode_query(const void *mode, int request, void *ptr);
+
 #endif
