@@ -1,7 +1,11 @@
-#include "speex.h"
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
+#include <speex/speex.h>
 #include <stdio.h>
 #include <stdlib.h>
-#include "speex_callbacks.h"
+#include <speex/speex_callbacks.h>
 
 #ifdef FIXED_DEBUG
 extern long long spx_mips;
